package net.openhft.chronicle.lataeg;

import net.openhft.lang.model.Byteable;

public interface IChronicleQueueData extends Byteable {

    int getIntField1();

    void setIntField1(int intField1);

    int getIntField2();

    void setIntField2(int intField2);

    int getIntField3();

    void setIntField3(int intField3);

    int getIntField4();

    void setIntField4(int intField4);

    int getIntField5();

    void setIntField5(int intField5);

    int getIntField6();

    void setIntField6(int intField6);

    int getIntField7();

    void setIntField7(int intField7);

    int getIntField8();

    void setIntField8(int intField8);

    int getIntField9();

    void setIntField9(int intField9);

    int getIntField10();

    void setIntField10(int intField10);

    int getIntField11();

    void setIntField11(int intField11);

    int getIntField12();

    void setIntField12(int intField12);

    int getIntField13();

    void setIntField13(int intField13);

    int getIntField14();

    void setIntField14(int intField14);

    int getIntField15();

    void setIntField15(int intField15);

    int getIntField16();

    void setIntField16(int intField16);

    int getIntField17();

    void setIntField17(int intField17);

    int getIntField18();

    void setIntField18(int intField18);

    int getIntField19();

    void setIntField19(int intField19);

    int getIntField20();

    void setIntField20(int intField20);

}