/*
 * Copyright 2015 Higher Frequency Trading
 *
 * http://www.higherfrequencytrading.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.openhft.chronicle.queue.stateless.bytes;

import net.openhft.chronicle.bytes.Bytes;
import org.jetbrains.annotations.NotNull;

import java.util.function.UnaryOperator;

/**
 * todo : currently work in process
 *
 * Created by Rob Austin
 */
public interface StatelessRawBytesAppender {

    /**
     * @param bytes append bytes to the tail
     * @return returns the index of the newly created except
     */
    long appendExcept(@NotNull Bytes bytes);

    /**
     * @param bytes  append bytes to offset
     * @param offset where the data should be written
     */
    void append(long offset, @NotNull Bytes bytes);

    void append(@NotNull UnaryOperator<Bytes> unaryOperator);
}
