/*
 * Copyright 2014 Higher Frequency Trading
 *
 * http://www.higherfrequencytrading.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.openhft.chronicle;

import org.junit.runner.Description;
import org.junit.runner.notification.RunListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ChronicleTestRunListener extends RunListener {
    private static final Logger LOGGER = LoggerFactory.getLogger("HFT");

    public void testStarted(Description description)   {
        if(Boolean.getBoolean("hft.test.printTestStarted")) {
            LOGGER.info("TestStarted >> {}::{}", description.getClassName(), description.getMethodName());
        }
    }

    public void testFinished(Description description)   {
        if(Boolean.getBoolean("hft.test.printTestFinished")) {
            LOGGER.info("TestFinished << {}::{}", description.getClassName(), description.getMethodName());
        }
    }
}
